<?php

namespace Drp\ProjectHelpers\Tests\Concerns;

/**
 * Class InteractsWithModels
 * @package Drp\ProjectHelpers
 */
trait InteractsWithModels
{
    /**
     * @param $expected
     * @param $result
     * @return $this
     */
    protected function assertSameModel($expected, $result)
    {
        $this->assertInstanceOf(get_class($result), $expected, 'Not the same model');
        $this->assertEquals($expected->id, $result->id, 'Not the same model');
        return $this;
    }

    /**
     * @param $expected
     * @param $result
     * @return $this
     */
    protected function assertNotSameModel($expected, $result)
    {
        $this->assertNotEquals($expected->id, $result->id, 'Is the same model');
        return $this;
    }

    /**
     * @param $expected
     * @param $result
     * @return $this
     */
    protected function assertContainsModel($expected, $result)
    {
        $this->assertTrue($this->containsModel($expected, $result));
        return $this;
    }

    /**
     * @param $expected
     * @param $result
     * @return $this
     */
    protected function assertNotContainsModel($expected, $result)
    {
        $this->assertFalse($this->containsModel($expected, $result));
        return $this;
    }

    /**
     * @param $expected
     * @param $result
     * @return mixed
     */
    private function containsModel($expected, $result)
    {
        return $result->contains(function ($model) use ($expected) {
            $class = get_class($model);
            return $expected instanceof $class && $model->id === $expected->id;
        });
    }
}
